# -*- coding: utf-8 -*-
'''
  product:  privacyIDEA
  module:   privacyidea weblib
    
  Copyright (c) 2014 Cornelius Kölbel
  License:  AGPLv3
  contact:  http://www.privacyidea.org
            
  Description:  The weblib is supposed to contain base functions that
                will only work with pylons

  Dependencies: -
'''